/******************************************************************************
 * File:		CharBuffer.h
 * Date:		Unknown
 * Author:		Unknown
 *
 * Copyright © 1996-1997 by Blue World Communications, Inc. All rights reserved.
 *
 * History:
 *		07.05.2000 - Made CHAR_EMPTY and exported symbol - CJS.
 *
 *
 *****************************************************************************/
 
/*********************************************************************

 WARNING!!!  DANGER!!!  WARNING!!!  DANGER!!!  WARNING!!!  DANGER!!!

	There is some very subtle code in this class and it's not well
	documented. This is a key class used by almost every other class in
	Lasso. IF YOU BREAK THIS CODE YOU WILL CAUSE YOURSELF MUCH HEARTACHE
	AND ANGUISH. The best thing is probably not to touch it. If you must,
	be very careful. Test your changes thoroughly. Always test this code
	with NDEBUG undefined (see prefix commands under Project Options);
	this will enable the numerous assert() calls that will help keep
	you out of trouble.
	
	BE AFRAID WHENEVER YOU TOUCH THIS CODE!  YOU HAVE BEEN WARNED.

 DANGER!!!  WARNING!!!  DANGER!!!  WARNING!!!  DANGER!!!  WARNING!!!

*********************************************************************/

/***

	MEMORY ALLOCATION NOTE

	The copy constructor and operator= will both allocate memory for
	the buffer if the buffer is NULL. This allows you to call the
	default constructor, which doesn't allocate memory for the buffer,
	and then allocate memory via assignment or copy..

	CopyFileContents() will allocate memory if b is NULL.

***/
#ifndef _CHARBUFFER_H_
#define _CHARBUFFER_H_




// Includes:
//
#include "LassoCAPI.h"
#include <string.h>						// strcpy, strcat, etc
#include <ctype.h>
#include <stdio.h>
#include <unicode/utf.h>

#ifndef NDEBUG
#	define ZERO_MEM
#endif // NDEBUG

#ifdef WIN32
#define isostricmp stricmp
#define isostrincmp strincmp
#define EXPORTCLASS __declspec(dllexport)
#define EXPORT __declspec(dllexport)
#pragma warning(disable: 4996)
#else
#define isostricmp strcasecmp
#define isostrincmp strcasencmp
#define EXPORTCLASS
#define EXPORT
#endif
// Constants:
//
const int32_t kStandardCharBufferSize = 255;
const int32_t kStandardCharBufferGrowSize = kStandardCharBufferSize;
const uint32_t kMaxFileReadSize = 16384;

const int32_t kUseGrowSizeThreshold = 2;

const int32_t kXlateFlagStrID = 10130;		// resource ID of translations flag

const int32_t kISOtoMacResID = 10128;		// ISO to Mac translation tables
const int32_t kMactoISOResID = 10129;

// User-defined types:
//
typedef char * (*AllocCharArrayProc)(int32_t num);
typedef void (*DisposeCharArrayProc)(char *c);

#if MACHINE_PC || MACHINE_UNIX
typedef unsigned char Str255[256];
#endif // MACHINE_PC || MACHINE_UNIX

/******************************************************************************
 Class:			CharBuffer
 Description:	Lasso's core string class.  This class is a C++ wrapper for a 
				standard C string.
 Notes:			Document all changes thouroghly.
 */
class EXPORTCLASS CharBuffer
{
	// Services:
	//
	public:
		/*virtual*/ ~CharBuffer();
	
		CharBuffer();
		CharBuffer(const char * s, int32_t theSize = -1);
		CharBuffer(const CharBuffer & buff);
		explicit CharBuffer(const UChar * s, int len = -1, const char * encoding = "UTF-8");
		CharBuffer(const UChar * s, int len, struct UConverter * conv);
		explicit CharBuffer(int32_t theSize);
	
	inline char * BuffPtr() const { return b; }
	inline const char * GetString() const { return b; }
	inline uint32_t ContentLen() const { return fLen; }
	inline uint32_t BufferSize() const { return fSize; }
	inline uint32_t Capacity() const { return (fSize - fLen); }
	inline void SetGrowSize(int32_t size) { fGrowSize = size; }
	
	inline int32_t	Hash() const;
	static inline int32_t HashString(const char *s);
	
	void StealBuffer(CharBuffer &from);
	char *ReleaseBuffer();
	void ClaimBuffer(char *buff, int32_t len = -1);
		
	inline void
	SetContentLen( int32_t length )
	{
//		ASSERT( length >= 0 && length <= (int32_t)fSize );
		if (length < 0)
			length = 0;
		else if ( length > (int32_t)fSize )
			length = fSize;
			
		fLen = length;
		if (b) b[ fLen ] = '\0';
	}

	void ExtractWord( int32_t *pos, CharBuffer *dest, char termChar = '\0' ) const;
	
	void GetPStr( Str255 pstr ) const;

	int32_t GetInt32() const;
	int64_t GetInt64() const;
	double GetDouble() const;

	bool Blank() const;
		
	int32_t Search( const char *s, int32_t start = 0, int32_t numChars = 0 ) const;
	int32_t Search( char s, int32_t start = 0, int32_t numChars = 0 ) const;
	int32_t RevSearch( char s, int32_t start = -1) const;
	
	// inserts a buffer, replacing previous contents
	void Insert( int32_t pos, const CharBuffer &s, int32_t startPos = 0, int32_t endPos = 0 );
	// inserts a buffer with the previous contents being shifted down
	void Merge( int32_t pos, const CharBuffer & s, int32_t startpos = 0, int32_t endPos = 0 );
	// remove a portion of the buffer. contents after the specified portion are shifted back
	void Remove( int32_t startPos, int32_t numChars = -1 );
	
	void Append( const char *s, uint32_t size );
	void Append( const CharBuffer &s, int32_t startPos, int32_t endPos );
	inline void Append( const CharBuffer &s ) { this->Append(s, 0, 0); }

	void Append( const char *s );
	void AppendNum( int32_t num );
	void Replace( char *subStr, CharBuffer *newStr );
	
	CharBuffer& LowerCase();
	CharBuffer& UpperCase();
	
	void RemoveWhiteSpace();	//Added 2/29/00 - SLH
	void CollapseWhiteSpace();
	
	CharBuffer& DecodeURL();
	CharBuffer& EncodeURL();
	CharBuffer& EncodeShittyURL();
//	CharBuffer& EncodeUTF();
//	CharBuffer& DecodeUTF();
	
	CharBuffer& EncodeHTML( bool convertBreaks = false, bool doBrackets = true );
	
	void ValidateHTTPHeader();
	void Reset();
	char GetChar( int32_t i ) const;
	
	inline bool Equals(const CharBuffer &buf) const
		{ return (*this) == buf; }
	inline bool Equals(const CharBuffer *buf) const
		{ return (*this) == *buf; }
	inline bool Equals(const char *c) const
		{ return (*this) == c; }
	
	bool EndsWith(const char * s) const;
	
	inline char & operator[]( int32_t i ) const
	{	
		static char CHAR_EMPTY = 0;
		if (!b) { return CHAR_EMPTY; }
		return b[i];
	}
#if defined(WIN32) && _MSC_VER <= 1200 // WIN32 doesn't mind that int and int32_t are the same
	inline char & operator[]( unsigned int i ) const { return this->operator[]((int32_t)i); }
	inline char & operator[]( int i ) const { return this->operator[]((int32_t)i); }
#endif
	inline char & operator[]( uint32_t i ) const { return this->operator[]((int32_t)i); }

	CharBuffer& operator=(int32_t num);
	CharBuffer& operator=(uint32_t num);
	CharBuffer& operator=(int16_t num);
	CharBuffer& operator=(uint16_t num);
	CharBuffer& operator=(int64_t num);
	CharBuffer& operator=(uint64_t num);
	CharBuffer& operator=(double d);
	CharBuffer& operator=(const char *s);
	CharBuffer& operator=(const CharBuffer &buff);
//	CharBuffer& operator=(const osVector<char> &theVector);
	
	inline CharBuffer& operator+=( char s ) 
	{
		if ( !b )
			GrowBufferSize(1);
		if ( fLen < fSize )
		{
			b[ fLen ] = s; 
			fLen++; 
			b[ fLen ] = '\0'; 
		}
		else
		{
			if ( fGrowSize > 0 )
			{
				GrowBufferSize( 1 );
				b[ fLen ] = s;
				fLen++;
				b[ fLen ] = '\0';
			}
		}
		return *this;
	}
	
	inline CharBuffer& operator+=( const char *s ) 
	{
		Append(s); 
		return *this; 
	}
	
	inline CharBuffer& operator+=( const CharBuffer &buff ) 
	{
		Append(buff); 
		return *this; 
	}
		
	inline CharBuffer& operator+=( int32_t num )
	{
		char x[128] = ""; 
		sprintf(x, "%d", (int)num); 
		return (*this)+= x; 
	}

	inline CharBuffer& operator+=(uint32_t num)
	{
		char x[128] = ""; 
		sprintf(x, "%u", (unsigned int)num); 
		return (*this)+= x; 
	}
		
	inline CharBuffer& operator+=(int16_t num)
	{
		char x[128] = ""; 
		sprintf(x, "%d", (int)num); 
		return (*this)+= x; 
	}
		
	inline CharBuffer& operator+=(uint16_t num)
	{
		char x[128] = ""; 
		sprintf(x, "%u", (unsigned int)num); 
		return (*this)+= x; 
	}
	
	inline CharBuffer & operator+=(int64_t l)
	{
		char x[128];
		sprintf(x, "%ld", (long int)l);
		return (*this) += x;
	}

	inline CharBuffer & operator+=(uint64_t l)
	{
		char x[128];
		sprintf(x, "%lu", (long unsigned int)l);
		return (*this) += x;
	}
	
	inline CharBuffer& operator<<( char s ) 
	{
		return (*this) += s; 
	}
	
	inline CharBuffer& operator<<( const char *s ) 
	{
		return (*this) += s; 
	}
	
	inline CharBuffer& operator<<( const CharBuffer &buff ) 
	{
		return (*this) += buff; 
	}
	
	inline CharBuffer& operator<<( int32_t num ) 
	{
		return (*this) += num; 
	}
	
	inline CharBuffer& operator<<(uint32_t num) 
	{ 
		return (*this) += num; 
	}

	inline CharBuffer& operator<<( int64_t num ) 
	{
		return (*this) += num; 
	}
	
	inline CharBuffer& operator<<(uint64_t num) 
	{ 
		return (*this) += num; 
	}

	inline CharBuffer& operator<<(int16_t num) 
	{ 
		return (*this) += num; 
	}
	
	inline CharBuffer& operator<<(uint16_t num) 
	{ 
		return (*this) += num; 
	}
	
	inline CharBuffer& operator<<(double f)
	{
		char x[128] = ""; 
		sprintf(x, "%f", f); 
		int32_t len = int32_t(strlen(x) - 1);
		while (x[len] == '0' && x[len - 1] != '.')
			x[len--] = '\0';
			
		return (*this)+= x;
	}
	
	inline bool operator==(const char *s) const
	{ 
		return isostricmp( b, s ) == 0; 
	}

	inline bool operator==(const CharBuffer& y) const
	{ 
		return isostricmp( b, y.b ) == 0; 
	}

	inline bool operator!=(const char *s) const
	{ 
		return !(*this == s); 
	}

	inline bool operator!=(const CharBuffer& y) const
	{ 
		return !(*this == y); 
	}

	inline bool operator<( const CharBuffer &y ) const
	{ 
		return isostricmp( b, y.b ) < 0; 
	}
	
	inline bool operator>( const CharBuffer & y ) const
	{
		return isostricmp( b, y.b ) > 0;
	}
		
	inline operator char*() const
	{ 
		return b; 
	}
		
	void GrowBufferSize( int32_t n );

protected:

	// Attributes:
	//
	protected:
		
		char *b;						// the buffer
		
		uint32_t fSize;			// buffer size
		uint32_t fLen;				// len of buffer contents
	protected:	
		
		int32_t fGrowSize;					// 

	inline void Init( int32_t growSize )
	{
		fLen  = 0;
		fSize = 0;
		b = NULL;

		fGrowSize    = growSize;
	}

	void	Copy( const char *s );
	
	void	IncreaseBufferSize( int32_t n );	

};	// end CharBuffer


inline int32_t
CharBuffer::Hash() const
{
	if ( !b )
		return 0;

	const char * s = b;
	uint32_t h = 0; 
	  for ( ; *s; ++s)
    		h = 5*h + tolower(*s);
	return int32_t(h);

//	long	n = ContentLen();
//	const char *d = BuffPtr();
//	long	h = 0;
//	
//	for (long i = 0; i < n; ++i, ++d)
//		h = (h << 2) + isotolower(*d);
//	return ((h >= 0) ? (h % prime) : (-h % prime));
}

inline int32_t
CharBuffer::HashString(const char *d)
{
	uint32_t h = 0; 
	  for ( ; *d; ++d)
    		h = 5*h + tolower(*d);
	return int32_t(h);

//	long	n = strlen(d);
//	long	h = 0;
//	
//	for (long i = 0; i < n; ++i, ++d)
//		h = (h << 2) + isotolower(*d);
//	return ((h >= 0) ? (h % prime) : (-h % prime));	
}




/******************************************************************************
 Class:			StBufCheck
 Description:	this will tell you if the content length ever exceeds the 
				buffer size (BAD!)
 Notes:			
 */

class EXPORTCLASS StBufCheck
{
	// Attributes:
	//
	protected:
		const CharBuffer & buf;			// reference to buffer

	// Services:
	//
	public:
		StBufCheck(const CharBuffer & b) : buf(b)
		{
//			ASSERT( b.BufferSize() >= b.ContentLen() );
		}	// end constructor

		~StBufCheck()
		{
//			ASSERT( buf.BufferSize() >= buf.ContentLen() );
		}	// end destructor

};	// end StBufCheck



#if DEBUG_BUILD
	#define CHAR_BOUNDS_CHECK(x) StBufCheck check(x)

#else
	#define CHAR_BOUNDS_CHECK(x)

#endif // DEBUG_BUILD

#endif // _CHARBUFFER_H_


///////////////////////////////// END OF FILE /////////////////////////////////
