/*
 *  LassoCAPI.h
 *  LassoC
 *
 *  Created by Kyle Jessup on 3/29/06.
 *  Copyright 2006 LassoSoft. All rights reserved.
 *
 */
/** \mainpage LassoCAPI
 *
 * This is the Doxygen reference for LassoCAPI 9.
 */

/** \file LassoCAPI.h
 * @brief LassoCAPI Main Header File
 *   
 * This file contains all of the available LCAPI defines and functions.
 */
#ifndef _C_LASSOCAPI_H_
#define _C_LASSOCAPI_H_

#include <unicode/uchar.h>

#ifdef WIN32
	#define	MACHINE_PC		1
	#define MACHINE_UNIX	0
#else
	#define	MACHINE_PC		0
	#define MACHINE_UNIX	1
#endif

#ifndef UCP
#define UCP(X) ((const UChar*)(L##X))
#endif

#define kLPOpBegin "-opbegin"
#define kLPOperatorBegin "-operatorbegin"
#define kLPOpEnd "-opend"
#define kLPOperatorEnd "-operatorend"

#define kLPOpBeginW LPW(L"-opbegin")
#define kLPOperatorBeginW LPW(L"-operatorbegin")
#define kLPOpEndW LPW(L"-opend")
#define kLPOperatorEndW LPW(L"-operatorend")

enum SortOrder { orderAscending, orderDescending, orderCustom = 4 };

/*
 Operators for datasources
 */
enum
{
	kLassoGreaterThan = '>   ',
	kLassoGreaterThanEquals = '>=  ',
	kLassoEquals = '=   ',
	kLassoLessThan = '<   ',
	kLassoLessThanEquals = '<=  ',
	kLassoBeginsWith = 'bgwt',
	kLassoEndsWith = 'ends',
	kLassoContains = 'cont',
	kLassoNotContains = '!cts',
	kLassoNotBeginsWith = '!bgs',
	kLassoNotEndsWith = '!end',
	kLassoAND = 'AND ',
	kLassoOR = 'OR  ',
	kLassoNOT = 'NOT ',
	kLassoNo = 'no  ',
	kLassoAny = 'any ',
	kLassoInList = ' nlt',
	kLassoNotInList = '!nlt',
	kLassoInFullText = ' ftx',
	kLassoInRegExp = ' rxp',
	kLassoNotInRegExp = '!rxp'
};

typedef enum lpEncodingType 
{ 
	encodeURL,							// TODO: Comment encoding methods here
	encodeRaw, 							// 
	encodeSmart, 						// 
	encodeBreak, 						// 
	encodeDefault, 						// 
	encodeStrictURL, 					// 
	encodeXML,
	encodeNone,							// 
	encodeHTML
} encodingMethod;	// end encodingMethod

const int kNumErrors = 71;

enum osError
{
	osErrNoErr = 0,
	osErrAssert = -10000,
	
	// stream errors
	osErrStreamReadError,
	osErrStreamWriteError,
	
	// memory errors
	osErrMemory,
	osErrInvalidMemoryObject,
	osErrOutOfMemory,
	osErrOutOfStackSpace,
	osErrCouldNotLockMemory,
	osErrCouldNotUnlockMemory,
	osErrCouldNotDisposeMemory,
	
	// file errors
	osErrFile,							// generic file error
	osErrFileInvalid,					// trying to work with an invalid file
	osErrFileInvalidAccessMode,			// trying to access a file in a mode that it doesn't support
	osErrCouldNotCreateOrOpenFile,		// could not create or open the file
	osErrCouldNotCloseFile,				// could not close the file
	osErrCouldNotDeleteFile,			// could not delete the file
	osErrFileNotFound,					// file does not exist
	osErrFileAlreadyExists,				// trying to create a file that already exists
	osErrFileCorrupt,					// file is corrupted
	osErrVolumeDoesNotExist,			// bad volume name
	osErrDiskFull,						// no room left on disk
	osErrDirectoryFull,					// no more items allowed inthe directory
	osErrIOError,						// I/O error
	osErrInvalidPathname,				// pathname is invalid
	osErrInvalidFilename,				// filename is invalid
	osErrFileLocked,					// file is locked
	osErrFileUnlocked,					// file is unlocked
	osErrFileIsOpen,					// file is open
	osErrFileIsClosed,					// file is closed
	osErrBOF,							// beginning of file reached
	osErrEOF,							// end of file reached
	osErrCouldNotWriteToFile,			// unable to complete a write operation to the file
	osErrCouldNotReadFromFile,			// unable to complete a read operation from the file
	
	// resource errors
	osErrResNotFound,
	osErrResource,
	
	// networking errors
	osErrNetwork,						// unknown networking error
	
	// security violation errors
	osErrInvalidUsername,				// the username supplied for the action is not valid
	osErrInvalidPassword,				// the password supplied for the action is not valid
	osErrInvalidDatabase,				// the database name supplied is not valid
	osErrNoPermission, 					// general permissions error
	osErrFieldRestriction, 				// the specified action is restricted
	
	// contiguous web errors
	osErrWebAddError,
	osErrWebUpdateError,
	osErrWebDeleteError,
	
	// general errors
	osErrInvalidParameter,				// an invalid parameter was passed to a function
	osErrOverflow,						// some memory passed to a function was too small to hold the results
	osErrNilPointer,					// a pointer was nil when it shouldn't have been
	osErrUnknownError,					// default when none of the cross-platform errors seem to fit
	
	osErrLoopAborted,			// a looping tag was aborted; all looping tags must catch this exception
	osErrSyntaxError,			// bad syntax used in a format file; parsing of the file was aborted	
	
	osErrDivideByZero,					
	osErrIllegalInstruction,
	osErrTagNotFound,
	osErrVarNotFound,
	
	osErrAborted,
	osErrFailure,
	
	osErrPreconditionFailed,
	osErrPostconditionFailed,
	osErrCriteriaNotMet,
	
	osErrIllegalUseOfFrozenInstance,
	osErrCompilationError,
	
	osErrNotImplemented,
	
	osErrSyntaxWarning,
	
	// NOTE: All errors that aren't contiguous with the above errors
	//		 should be added below.  Also, any new error messages in the continguous
	//		 area should be added between osErrAssert and osErrUnknownError.
	
	// non contiguous web errors (backwards compatible)
	osErrWebRequiredFieldMissing  = -800,		// value missing for required field for Add
	osErrWebRepeatingRelatedField = -801,		// Adding repeating related fields isn't supported
	osErrWebNoSuchObject = -1728,				// same error number as errAENoSuchObject
	osErrWebTimeout = -1712,					// same error number as errAETimeout
	osErrWebActionNotSupported = -802,
	osErrConnectionInvalid = -609,			// same error number as connectionInvalid
	osErrWebModuleNotFound = -2000,
	osErrHTTPFileNotFound = 404,
	osErrDatasourceError = -3000				// third-party generic datasource error
	
};

#if !MACHINE_LINUX
#ifdef FD_SETSIZE
#undef FD_SETSIZE
#endif
#define FD_SETSIZE 128
#endif

#define LPW(X) ((const UChar*)X)

typedef unsigned int LP_TypeDesc;

namespace LPTypes
{
const LP_TypeDesc lpTypeString		= 'TEXT';
const LP_TypeDesc lpTypeNull		= 'null';
const LP_TypeDesc lpTypeInteger		= 'long';
const LP_TypeDesc lpTypeBoolean		= 'bool';
const LP_TypeDesc lpTypeBytes		= 'blob';
const LP_TypeDesc lpTypeDecimal		= 'doub';
const LP_TypeDesc lpTypeDateTime    = 'Date';
const LP_TypeDesc lpTypeArray		= 'arry';
const LP_TypeDesc lpTypeTag			= 'code';
const LP_TypeDesc lpTypePair		= 'pair';
const LP_TypeDesc lpTypeCustom		= 'Yers';

const LP_TypeDesc kDateDataType	= 'Date';
const LP_TypeDesc kProtectionNone = 'none';
const LP_TypeDesc kProtectionReadOnly = 'nmod';
}

using namespace LPTypes;

extern "C"
{

#ifdef WIN32

#ifdef EXPORT_LCAPI /* external modules don't define anything */
#define LCAPICALL __declspec(dllexport)
#else
#define LCAPICALL __declspec(dllimport)
#endif

#else

#ifdef EXPORT_LCAPI /* external modules don't define anything */
#define LCAPICALL // automatically all exported
#else
#define LCAPICALL extern
#endif

#endif

#define LCAPI_DEPRECATED

/**
	Defines for testing the LCAPI version
 */
#define LCAPI_VERSION 9   /**< Current Lasso Version. */
#define LCAPI_VERSION_1 1 /**< Lasso Version 5. First LCAPI release */
#define LCAPI_VERSION_2 2 /**< Lasso Version 6. Second LCAPI release */
#define LCAPI_VERSION_3 3 /**< Lasso Version 7. Third LCAPI release */
#define LCAPI_VERSION_4 4 /**< Lasso Version 8. Fourth LCAPI release */
#define LCAPI_VERSION_9 LCAPI_VERSION /**< Lasso Version 9 */

/** 
	@brief Used for retrieving data from some LCAPI functions.
	
	The lasso_value_t struct is used for shuffling data 
	into and out of LassoCAPI functions.

	Any LCAPI function that constructs one of these will insure that Lasso properly 
	disposes of the data once the user's LassoCAPI function call is complete.
	
	lasso_value_t is suitable for transferring binary data as long as the \a nameSize or \a dataSize
	members are properly set with the size of the data in bytes.
	
	Depending on the purpose of the function which is constructing the lasso_value_t, any combination 
	of the \a name or \a data members may be filled in or may be NULL. Consult the documentation of 
	the specific function for more details.
	
	Should be initialized using the INITVAL(X) macro before use.
*/
struct lasso_value_t
{
	/**
	 *	@brief The \a name portion.
	 *
	 *	May be NULL.
	 */
	const char * name;
	/**
	 *	@brief The size of the \a name member, in bytes.
	 *
	 *
	 */
	unsigned int nameSize;
	/**
	 *	@brief The \a value portion.
	 *
	 *	May be NULL.
	 */
	const char * data;
	/**
	 *	@brief The size of the \a data member, in bytes.
	 *
	 *
	 */
	unsigned int dataSize;
	/**
	 *	@brief The type code for the \a data member.
	 *
	 *
	 */
	LP_TypeDesc type;
};

/**
	@brief Used for retrieving Unicode character data from some LCAPI functions.
	
	The lasso_value_w_t struct is used for shuffling Unicode character data 
	into and out of LassoCAPI functions.

	Any LCAPI function that constructs one of these will insure that Lasso properly 
	disposes of the data once the user's LassoCAPI function call is complete.
	
	Depending on the purpose of the function which is constructing the lasso_value_t, any combination 
	of the \a name or \a data members may be filled in or may be NULL. Consult the documentation of 
	the specific function for more details.
	
	Should be initialized using the INITVAL(X) macro before use.
*/
struct lasso_value_w_t
{
	/**
	 *	@brief The \a name portion.
	 *
	 *	May be NULL.
	 */
	const UChar * name;
	/**
	 *	@brief The size of the \a name member, in bytes.
	 *
	 *
	 */
	unsigned int nameSize;
	/**
	 *	@brief The \a value portion.
	 *
	 *	May be NULL.
	 */
	const UChar * data;
	/**
	 *	@brief The size of the \a data member, in bytes.
	 *
	 *
	 */
	unsigned int dataSize;
	/**
	 *	@brief The type code for the \a data member.
	 *
	 *
	 */
	LP_TypeDesc type;
};

/**
	@brief Initialize a lasso_value_t or lasso_value_w_t to be blank.
	
	It is recommended that each lasso_value_t be initialized using this macro before use.
	
	Example:
	<pre>
		lasso_value_t myVal;
		INITVAL(&myVal);
	</pre>
*/
#define INITVAL(X) { (X)->name = (X)->data = NULL; (X)->nameSize = (X)->dataSize = 0; }

/**
	Special typedef so programmers know when Lasso will automatically dispose of the value.
*/
typedef lasso_value_t auto_lasso_value_t;
/**
	Special typedef so programmers know when Lasso will automatically dispose of the value.
*/
typedef lasso_value_w_t auto_lasso_value_w_t;

/**
	@brief Allocates a lasso_value_t with the indicated data.
	
	Anything allocated with this function will NOT be garbage collected by Lasso
	and must be freed using lasso_freeValue.
	
	@param result A pointer to the lasso_value_t which will be constructed.
	@param name The name portion which will be copied and set.
	@param nameSize The size of the name parameter in bytes.
	@param data The data portion which will be copied and set.
	@param dataSize The size of the data parameter in bytes.
	@param type The type code for the value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_allocValue( lasso_value_t * result, const char * name, unsigned int nameSize, const char * data, unsigned int dataSize, LP_TypeDesc type );
/**
	@brief Allocates a lasso_value_w_t with the indicated data.
	
	Anything allocated with this function will NOT be garbage collected by Lasso
	and must be freed using lasso_freeValueW.
	
	@param result A pointer to the lasso_value_w_t which will be constructed.
	@param name The name portion which will be copied and set.
	@param nameSize The number of characters in the name portion.
	@param data The data portion which will be copied and set.
	@param dataSize The number of characters in the data portion.
	@param type The type code for the value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_allocValueW( lasso_value_w_t * result, const UChar * name, unsigned int nameSize, const UChar * data, unsigned int dataSize, LP_TypeDesc type );
/**
	@brief Allocates a lasso_value_t with the indicated data and encoding method.
	
	This function can be used to convert Unicode data into any of the supported encodings.
	
	Anything allocated with this function will NOT be garbage collected by Lasso
	and must be freed using lasso_freeValue.
	
	@param result A pointer to the lasso_value_w_t which will be constructed.
	@param name The name portion which will be converted and set.
	@param nameSize The number of characters in the name portion.
	@param nameEncoding The destination encoding for the name portion.
	@param data The data portion which will be converted and set.
	@param dataSize The number of characters in the data portion.
	@param dataEncoding The destination encoding for the data portion.
	@param type The type code for the value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_allocValueConv( lasso_value_t * result, const UChar * name, unsigned int nameSize, const char * nameEncoding, 
																const UChar * data, unsigned int dataSize, const char * dataEncoding, LP_TypeDesc type );
/**
	@brief Frees a previously allocated lasso_value_t
	
	Do not pass an auto_lasso_value_t to this function or you will end up with a double free.
	
	@param result The lasso_value_t whose members will be freed.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_freeValue( lasso_value_t * result );
/**
	@brief Frees a previously allocated lasso_value_w_t
	
	Do not pass an auto_lasso_value_w_t to this function or you will end up with a double free.
	
	@param result The lasso_value_w_t whose members will be freed.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_freeValueW( lasso_value_w_t * result );


/*********************************************************
	DATASOURCE ACTIONS
**********************************************************/
/**
	@brief Types of actions datasources could be called for.
	
	One of these will be passed as the \a action parameter for the lasso_ds_func.
*/
enum datasource_action_t
{
	datasourceInit,			/**< Sent when a new "instance" of the datasource module is created. */
	datasourceTerm,			/**< Sent when the "instance" of the datasource module is destroyed. */
	datasourceExists,		/* UNUSED */

	datasourceNames,		/**< Sent when Lasso attempts to gather the names of all the databases that the datasource module supports.
							 	 Call lasso_addDataSourceResult once for each supported database.
							 */
	datasourceTableNames,   /**< Sent when Lasso attempts to gather the names of the tables available for the given database.
								 The name of the database itself will be passed in the \a param parameter.
							 	 Call lasso_addDataSourceResult once for each available table.
							 */
	datasourceSearch,		/**< Sent when the datasource module is to perform a -search action. */ 
	datasourceAdd,			/**< Sent when the datasource module is to perform a -add action. */
	datasourceUpdate,		/**< Sent when the datasource module is to perform a -update action. */
	datasourceDelete,		/**< Sent when the datasource module is to perform a -delete action. */
	datasourceInfo,			/**< Sent when the datasource module is to perform a -show action. */
	datasourceExecSQL,		/**< Sent when the datasource module is to perform a -sql action. */

	datasourceRandom,		/**< Sent when the datasource module is to perform a -random action. */

	datasourceSchemaNames, 	/**< Sent when Lasso attempts to gather the names of the schemas available for the given database.
								 The name of the database itself will be passed in the \a param parameter.
							 	 Call lasso_addDataSourceResult once for each available schema.
							 	 
							 	 This is currently only utilized for LJAPI.
							 */

	datasourceCloseConnection, /**< Sent when the datasource module should close a connection previously 
									set via the lasso_setDSConnection function. 
								*/
	datasourceTickle, 		/**< Sent to the datasource module when -database and -table are specified in an inline, 
							     but no action, or a -nothing action is used.
							     The database could, perhaps, set or reset its connection to the database via lasso_setDSConnection.
							     Or, it could do nothing.
							 */
	datasourceDuplicate,	// FileMaker module ONLY impliments this message
	datasourceScripts,		// FileMaker module ONLY impliments this message
	datasourceImage,		// FileMaker module ONLY impliments this message

	datasourceFindAll,		/**< Sent when the datasource module is to perform a -findall action. */
	datasourceMatchesName,	/**< Sent to the datasource to find out if it "goes" under the given name. 
								 The name which is being tested will be passed in the name member of the \a param parameter.
								 The data member of the \a param parameter will be NULL and the dataSize member will be zero.
								 If the name matches the name the datasource goes under, the datasource 
								 should both set the dataSize member to non-zero and return osErrNoErr. 
								 Otherwise, it is assumed that the name is not a matching name for the datasource module.
							*/
	datasourcePrepareSQL,	/**< Sent to the datasource to prepare a sql statement for later execution.
								 This is sent when the datasource action was -prepare. The statement text is sent to the
								 datasource in the \a data member of the \a param parameter that is passed to each datasource call.
							*/
	datasourceUnprepareSQL,  /**< Sent to the datasource after a datasourcePrepareSQL action.
								 This permits the datasource to perform and necessary cleanup activities after executing a prepared statement.
							*/
	datasourceMAXIMUM,
	datasourceNothing = -1
};

#define SET_MATCHED_NAME(param) { const_cast<auto_lasso_value_t*>(param)->dataSize = 1; }

/*********************************************************
	TAG ACTIONS
**********************************************************/
/**
	@brief Types of actions tag modules could be called for.
	
	Ignore this for now, it may be put into use in a future version but is not utliized at present.
*/
typedef int tag_action_t;

/*********************************************************
	LASSO REQUEST TOKEN/DATASOURCE TOKEN
**********************************************************/
/**
	@brief Special value passed to modules that identify the request.
	
	The same value should be used when calling into any LassoCAPI function.
*/
typedef struct lasso_request_t_{} * lasso_request_t;

/*********************************************************
	LASSO TYPE TOKEN - LCAPI 6
**********************************************************/
/**
	@brief Represents a type within Lasso
	
	This opaque value represents an instance of a type within Lasso
*/
typedef struct lasso_type_t_{} * lasso_type_t;

/*********************************************************
	DATASOURCE CONNECTION TOKEN - LCAPI 7
**********************************************************/
/**
	@brief Represents a datasource module connection.
	
	This opaque value is only interpreted by the datasource module itself.
	It can be stored via lasso_setDSConnection and retrieved via lasso_getDSConnection.
	Lasso will automatically send the datasource the datasourceCloseConnection message when it is time to close
	the connection.
*/
typedef struct lasso_dsconnection_t_{} * lasso_dsconnection_t;

/*********************************************************
	MODULE SERVICE FUNCTIONS
**********************************************************/
/**
	@brief Service function for Lasso Datasource modules
	
	A LCAPI datasource module should implement one of these and pass it to the lasso_registerDSModule or lasso_registerDSModuleW function 
	when the datasource module is registered. The function will be called by Lasso when a datasource operation is to be performed.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param action The current action for the datasource module to perform.
	@param param Any additional data associated with the action.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
typedef osError (*lasso_ds_func)(lasso_request_t token, datasource_action_t action, const auto_lasso_value_t * param);
/**
	@brief Service function for a Lasso tag
	
	An LCAPI tag should implement one of these and pass it to the lasso_registerTagModule or lasso_registerTagModuleW function
	when the LCAPI module is registered. The function will be called by Lasso every time the tag is called in a script.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param action Unused. Do not rely on this parameter to hold any particular value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
typedef osError (*lasso_tag_func)(lasso_request_t token, tag_action_t action);

/*********************************************************
	MODULE REGISTRATION
**********************************************************/
/**
	@brief LCAPI module registration callback.
	
	All Lasso modules must export a function named "registerLassoModule".
	When Lasso attempts to load an LCAPI module, it will look for this exported funtion and, if found, call it.
	Within that function, the user may register any number of datasource modules or any number of tags using the 
	lasso_registerDSModule, lasso_registerDSModuleW, lasso_registerTagModule or lasso_registerTagModuleW functions.
*/
typedef void (*register_module_func)(void);

/** @name Tag Registration Flags 

	The following flags may be OR'd together in various combinations and passed to lasso_registerTagModule or lasso_registerTagModuleW
	as the \a flags parameter to control how the tag behaves when it is called.
*/
/*@{*/
const int flag_typeInitializer =		0x00000001; /**< @brief The tag is to be treated as an initializer for a custom type. */
const int flag_typeSubstitutionTag =	0x00000002; /**< @brief The tag is to be treated as a "regular" tag. 
						
														 This is the default behavior so this flag is not required. 
													 */
const int flag_typeAsync =				0x00000004; /**< @brief The tag is to be run in its own thread. 

														 It will return no value to the caller. 
													 */
const int flag_typeContainerTag =		0x00000008; /**< @brief The tag is a container tag.

														 The tag must be called using the proper <pre>tag; ... /tag;</pre> syntax or an error is generated.
														 The result of executing any body statements can be retrieved using the lasso_childrenRun function.
													 */
const int flag_typeInterstitial =		0x00000010; /**< @brief Used internally. */
const int flag_typeSkipSecurityCheck =	0x00000040; /**< @brief Bypass any security checks for the tag. 

														 Useful to avoid any performance hits associated with security checks. 
														 Should only be used when a tag performs only trival or completely secure operations. 
													 */
const int flag_INTERNALONLY =			0x00000080; /**< @brief Used internally. */
const int flag_typeLoopingTag =			0x00000100; /**< @brief The tag is a looping container tag. 

														 For any tag registered with this flag, Lasso will automatically 
														 keep track of the loop_count and increment it each time the tag calls lasso_childrenRun.
													 */
const int flag_typeEval =				0x00000200; /**< @brief Used internally. Not useful for LCAPI tag types. */
const int flag_noGlobalImport =			0x00000400; /**< @brief The tag should not be automatically imported into the global namespace.

														 All LCAPI tags, starting with LP8, which are registered with a namespace should specify this flag.
													 */
const int flag_INTERNALONLY2 =			0x00000800; /**< @brief Used internally. */
const int flag_deprecated =				0x00001000; /**< @brief Use of the tag is deprecated. 

														 Depending on the administrator's configuration, calling the tag will 
														 automatically output a warning to the logging system.
													 */
const int flag_noDefaultEncoding =		0x00002000; /**< @brief Never apply default HTML encoding to the tag's return value. */
const int flag_INTERNALONLY3 =			0x00004000;
const int flag_prototype =				0x00008000; /**< @brief Applies only to types registered with lasso_registerTypeModule.
														
														An instance of the type will be created immediately (onCreate will not be called).
														Any further calls to this type will result in a fast copy of the prototype being created.
													*/
const int flag_atomic =					0x00010000;
const int flag_private =				0x00020000;

const int flag_nonBlockingCAPI =		0x00040000; // if not set, the capi call is run in its own thread to avoid blocking the physical host thread
										// INTERNAL NOTE
										// the flag above matches FLAG_RESERVED3
										//
										
// feel free to use these flags with lasso_typeXXXFlag
const int flag_User1 =					0x01000000; /**< @brief User defined flag.

														 May be used with lasso_tagSetFlag, lasso_tagHasFlag, lasso_tagClearFlag OR
														 lasso_typeSetFlag, lasso_typeHasFlag, lasso_typeClearFlag
													 */
const int flag_User2 =					0x02000000; /**< @brief User defined flag.

														 May be used with lasso_tagSetFlag, lasso_tagHasFlag, lasso_tagClearFlag OR
														 lasso_typeSetFlag, lasso_typeHasFlag, lasso_typeClearFlag
													 */
const int flag_User3 =					0x03000000; /**< @brief User defined flag.

														 May be used with lasso_tagSetFlag, lasso_tagHasFlag, lasso_tagClearFlag OR
														 lasso_typeSetFlag, lasso_typeHasFlag, lasso_typeClearFlag
													 */
const int flag_User4 =					0x04000000; /**< @brief User defined flag.

														 May be used with lasso_tagSetFlag, lasso_tagHasFlag, lasso_tagClearFlag OR
														 lasso_typeSetFlag, lasso_typeHasFlag, lasso_typeClearFlag
													 */
/** 
	@brief Recommended default registration flags.
	
	For a normal tag.
*/
const int REG_FLAGS_TAG_DEFAULT = (flag_typeSubstitutionTag | flag_noGlobalImport);
/** 
	@brief Recommended default registration flags.
	
	For a container tag.
*/
const int REG_FLAGS_CONTAINER_DEFAULT = (flag_typeContainerTag | flag_noGlobalImport);
/** 
	@brief Recommended default registration flags.
	
	For a looping container tag.
*/
const int REG_FLAGS_LOOPING_DEFAULT = (flag_typeLoopingTag | flag_noGlobalImport);
/** 
	@brief Recommended default registration flags.
	
	For a type initializer tag.
*/
const int REG_FLAGS_TYPE_DEFAULT = (flag_typeInitializer | flag_noGlobalImport);
/*@}*/

/**
	Registers a new tag.
	
	@param namespaceName The namespace in which the tag will be registered.
	@param tagName The name which the tag can be called by.
	@param func The user supplied callback for the tag.
	@param flags Any special flags for the tag.
	@param description A description for the tag.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_registerTagModule( const char * namespaceName, const char * tagName, lasso_tag_func func, int flags, const char * description );
LCAPICALL osError lasso_registerTagModuleW( const UChar * namespaceName, const UChar * tagName, lasso_tag_func func, int flags, const UChar * description );
/*@}*/

/** @name Type Registration 
	
	These functions are called to register a new type.
*/
/*@{*/
/**
	Registers a new type.
	
	@param namespaceName The namespace in which the type will be registered.
	@param tagName The name which the type can be called by.
	@param func The user supplied callback for the type initializer.
	@param flags Any special flags for the type. If flag_typeInitializer is ommited, it will be automatically added. Any conflicting flags such as flag_typeContainerTag or flag_typeLoopingTag will be ignored.
	@param description A description for the type.
	@param superType Optionally, the name of the new type's super type.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_registerTypeModule( const char * namespaceName, const char * tagName, lasso_tag_func func, int flags, const char * description, const char * superType );
LCAPICALL osError lasso_registerTypeModuleW( const UChar * namespaceName, const UChar * tagName, lasso_tag_func func, int flags, const UChar * description, const UChar * superType );
/*@}*/

/** @name Datasource Module Registration 
	
	These functions are called to register a new datasource module.
*/
/*@{*/
/**
	Registers a new datasource module.
	
	@param moduleName The name of the datasource module.
	@param func The User supplied callback for the datasource module.
	@param flags Any special flags for the datasource module.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_registerDSModule( const char * moduleName, lasso_ds_func func, int flags );
LCAPICALL osError lasso_registerDSModuleW( const UChar * moduleName, lasso_ds_func func, int flags );
LCAPICALL osError lasso_registerDSModule2( const char * moduleName, lasso_ds_func func, int flags, void * userData, void (*userDataDtor)(void*));
LCAPICALL osError lasso_registerDSModule2W( const UChar * moduleName, lasso_ds_func func, int flags, void * userData, void (*userDataDtor)(void*));

/*@}*/

LCAPICALL osError lasso_typeGetTrait( lasso_request_t token, lasso_type_t from, lasso_type_t * into );
/**	
	Set result message string.

	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param msg The result message string.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_setResultMessage( lasso_request_t token, const char * msg );
/**	
	Set result message string.

	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param msg The result message string.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_setResultMessageW( lasso_request_t token, const UChar * msg );

/**
	@brief Allocates a new type instance.
	
	The name of the type to allocate is signified by the second parameter. If a type initializer is found for the given name, it will be executed.
	An array of lasso_type_t parameters can be passed to the type initializer.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param typeName The name of the type to instantiate.
	@param paramCount The number of lasso_type_t parameters to pass to the initializer.
	@param paramsArray An array of parameters to pass to the type initializer.
	@param outType A pointer to the newly instantiated type.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAlloc ( lasso_request_t token, const char * typeName, int paramCount, lasso_type_t * paramsArray, lasso_type_t * outType );
/**
	@brief Allocates a new type instance.
	
	The name of the type to allocate is signified by the second parameter. If a type initializer is found for the given name, it will be executed.
	An array of lasso_type_t parameters can be passed to the type initializer.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param typeName The name of the type to instantiate.
	@param paramCount The number of lasso_type_t parameters to pass to the initializer.
	@param paramsArray An array of parameters to pass to the type initializer.
	@param outType A pointer to the newly instantiated type.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAllocW ( lasso_request_t token, const UChar * typeName, int paramCount, lasso_type_t * paramsArray, lasso_type_t * outType );
/**
	@brief Allocates a custom type within a type initializer.
	
	This function is used within lasso_tag_funcs that were registered as being a type initializer (flag_typeInitializer).
	It initializes a blank custom type and sets the type's name to the provided value.
	The new type does not yet have a lineage and has no members added to it.
	New data or tag members should be added using lasso_typeAddMember.
	The new type must be the return value of the tag call, set via lasso_returnTagValue.
	Any inherited members will be added to the type after the LCAPI call returns.
	
 	@warning Do NOT call this unless you are in a type initializer. If you are not in a type initializer, the result will be a type that will never be fully initialized.

	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param outCustom The resulting newly allocated custom type.
	@param name The new type's name.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAllocCustom( lasso_request_t token, lasso_type_t * outCustom, const char * name );
/**
	@brief Allocates a custom type within a type initializer.
	
	This function is used within lasso_tag_funcs that were registered as being a type initializer (flag_typeInitializer).
	It initializes a blank custom type and sets the type's name to the provided value.
	The new type does not yet have a lineage and has no members added to it.
	New data or tag members should be added using lasso_typeAddMember.
	The new type must be the return value of the tag call, set via lasso_returnTagValue.
	Any inherited members will be added to the type after the LCAPI call returns.
	
 	@warning Do NOT call this unless you are in a type initializer. If you are not in a type initializer, the result will be a type that will never be fully initialized.

	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param outCustom The resulting newly allocated custom type.
	@param name The new type's name.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAllocCustomW( lasso_request_t token, lasso_type_t * outCustom, const UChar * name );

/**
	@brief Allocate a new type instance based on the given type instance.
	
	The given type's tag members will be referenced in the new type.
	No data members are added to the new type.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param proto The original type instance whose tag members will be referenced.
	@param out The resulting new type instance.	
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAllocFromProto( lasso_request_t token, lasso_type_t proto, lasso_type_t * out );

/**
	@name Allocating Built-in Type Instances
	
	The following functions allocate instances of specific built-in types. 
	The lasso_request_t token may be null. If it is null, the allocated types are created as "detached" and must be manually freed using lasso_typeFree.
*/
/** @{ */
/**
	Allocates a new instance of type null.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param outNull The resulting new type instance.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAllocNull ( lasso_request_t token, lasso_type_t * outNull );
/**
	Allocates a new instance of type void.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param outVoid The resulting new type instance.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAllocVoid ( lasso_request_t token, lasso_type_t * outVoid );
/**
	Allocates a new instance of type string.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param outString The resulting new type instance.
	@param value The UTF-8 data from which to copy for the new string instance.
	@param length The length of the UTF-8 data in characters.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAllocString ( lasso_request_t token, lasso_type_t * outString, const char * value, int length );
/**
	Allocates a new instance of type string using the specified conversion method.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param outString The resulting new type instance.
	@param value The source data from which to copy for the new string instance.
	@param length The length of the source data in bytes.
	@param conv The encoding of the source data.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAllocStringConv ( lasso_request_t token, lasso_type_t * outString, const char * value, int length, const char * conv );
/**
	Allocates a new instance of type string.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param outString The resulting new type instance.
	@param value The UTF-16 data from which to copy for the new string instance.
	@param length The length of the UTF-16 data in characters.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAllocStringW ( lasso_request_t token, lasso_type_t * outString, const UChar * value, int length );
LCAPICALL osError lasso_typeAppendStringW( lasso_request_t token, lasso_type_t type, const UChar * val, int len );
/**
	Allocates a new instance of type integer.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param outInteger The resulting new type instance.
	@param value The integer value which the new type instance will hold.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAllocInteger ( lasso_request_t token, lasso_type_t * outInteger, int64_t value );
/**
	Allocates a new instance of type decimal.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param outDecimal The resulting new type instance.
	@param value The double value which the new type instance will hold.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAllocDecimal ( lasso_request_t token, lasso_type_t * outDecimal, double value );
/**
	Allocates a new instance of type decimal.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param outDecimal The resulting new type instance.
	@param value The double value which the new type instance will hold.
	@param precision The decimal precision that the new type instance will output to.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAllocDecimal2 ( lasso_request_t token, lasso_type_t * outDecimal, double value, int precision );
/**
	Allocates a new instance of type pair.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param outPair The resulting new type instance.
	@param first The instance to use for the first member of the pair. A reference to the instance will be made.
	@param second The instance to use for the second member of the pair. A reference to the instance will be made.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAllocPair ( lasso_request_t token, lasso_type_t * outPair, lasso_type_t first, lasso_type_t second );
/**
	Allocates a new hard reference to a type instance. The new instance will point to the original.
	This is a no-op under Lasso 9.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param outRef The resulting new type instance.
	@param referenced The instance to be referenced.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAllocReference( lasso_request_t token, lasso_type_t * outRef, lasso_type_t referenced );
/**
	Allocates a new instance of type tag.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param outTag The resulting new type instance.
	@param nativeTagFunction The LCAPI lasso_tag_func which will be called when the tag is used.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAllocTag ( lasso_request_t token, lasso_type_t * outTag, lasso_tag_func nativeTagFunction );
/**
	Allocates a new instance of type tag from the given source text.
	
	If the source text is UTF-8, it must contain a BOM or it will be treated as the default platform encoding.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param outTag The resulting new type instance.
	@param source The source text which will be compiled to become the body of the tag.
	@param length The length of the source text in characters.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAllocTagFromSource ( lasso_request_t token, lasso_type_t * outTag, const char * source, int length );
/**
	Allocates a new instance of type array.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param outArray The resulting new type instance.
	@param count The number of new array elements to insert. The number of items in the \a elements parameter.
	@param elements A pointer to an array of lasso_type_t which will become the elements of the array. Each item will be referenced.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAllocArray ( lasso_request_t token, lasso_type_t * outArray, int count, lasso_type_t * elements );
/**
	Allocates a new instance of type boolean.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param outBool The resulting new type instance.
	@param inValue The boolean value which the new type instance will hold.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAllocBoolean( lasso_request_t token, lasso_type_t * outBool, bool inValue );
/** @} */

/**
	@name Getting or Setting Values of Built-in Type Instances
	
	The following functions get and set the data on a previously created built-in type instance.
	When getting a value, the source type instance will not be altered.
	When setting a value, the provided type instance is converted, if required.
	The lasso_request_t token may be null.
*/
/** @{ */
/**
	Retrieves character data from a type instance.
	
	If the type is a string instance, the data will be converted to UTF-8.
	If the type is a bytes instance, the data will be provided unaltered.
	For any other type, the result will be the same as converting the type into a string and returning the data as UTF-8.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param type The type instance from which to retrieve the character data.
	@param val A pointer to a lasso_value_t struct in which to store the result. The data will be stored in the \a name portion of the struct.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeGetString( lasso_request_t token, lasso_type_t type, auto_lasso_value_t * val );
/**
	Retrieves character data from a type instance using the specified conversion method.
	If the special conversion method of "BINARY" is used, and the source type is a string, the resulting data will be UTF-16 data.
	If the special conversion method of "BINARY" is used, and the source type is bytes, the resulting data will be provided unaltered.
	For any other type, if the special conversion method of "BINARY" is used, the result will be the same as converting the instance 
	to a string and retrieving the UTF-16 data.

	The remaining possible values for the conversion method are any of the character encoding methods supported by ICU or any of the converters 
	stored in the '__external_converters__' global variable.
	
	In all cases, the \a nameSize portion of the resulting value struct will be the number of bytes in the result data.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param type The type instance from which to retrieve the character data.
	@param val A pointer to a lasso_value_t struct in which to store the result. The data will be stored in the \a name portion of the struct.
	@param conv The encoding method to use when transforming the Unicode string data.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeGetStringConv( lasso_request_t token, lasso_type_t type, auto_lasso_value_t * val, const char * conv );
/**
	Retrieves Unicode character data from a type instance.
	
	For any type other than string, the result will be the same as converting the type into a string and returning the data as UTF-16.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param type The type instance from which to retrieve the character data.
	@param val A pointer to a lasso_value_w_t struct in which to store the result. The data will be stored in the \a name portion of the struct.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeGetStringW( lasso_request_t token, lasso_type_t type, auto_lasso_value_w_t * val );
/**
	Retrieves the integer value from a type instance.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param type The type instance from which to retieve the integer value.
	@param out A pointer to the resulting 64-bit integer.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeGetInteger( lasso_request_t token, lasso_type_t type, int64_t * out );
/**
	Retrieves the decimal value from a type instance.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param type The type instance from which to retieve the decimal value.
	@param out A pointer to the resulting double.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeGetDecimal( lasso_request_t token, lasso_type_t type, double * out );
/**
	Retrieves the boolean value from a type instance.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param type The type instance from which to retieve the boolean value.
	@param out A pointer to the resulting boolean.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeGetBoolean( lasso_request_t token, lasso_type_t type, bool * out );
/**
	Converts the type instance into a string and sets the value.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param type The type instance to set.
	@param val The source data in UTF-8 encoding.
	@param len The length of the source UTF-8 data.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeSetString( lasso_request_t token, lasso_type_t type, const char * val, int len );
/**
	Converts the type instance into a string and sets the value.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param type The type instance to set.
	@param val The source data.
	@param len The length of the source data.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeSetStringW( lasso_request_t token, lasso_type_t type, const UChar * val, int len );
/**
	Retrieves the size of the provided array instance.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param array The array instance.
	@param len A pointer in which to store the resulting size.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_arrayGetSize( lasso_request_t token, lasso_type_t array, int * len );
/**
	Retrieves the specified element of the provided array instance.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param array The array instance.
	@param index The zero based index at which to retreive.
	@param out The pointer in which to store the result.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_arrayGetElement( lasso_request_t token, lasso_type_t array, int index, lasso_type_t * out );
/**
	Sets the specified element of the provided array instance.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param array The array instance.
	@param index The zero based index at which to set. If the index is greater than the array's current size or is less than zero, the new item is added to the end of the array.
	@param elem The type instance which will be placed at the specified index. The instance will be referenced.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_arraySetElement(lasso_request_t token, lasso_type_t array, int index, lasso_type_t elem );
/**
	Retrieves the first member from the provided pair.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param pr The pair instance.
	@param out The pointer in which to store the result.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_pairGetFirst( lasso_request_t token, lasso_type_t pr, lasso_type_t * out );
/**
	Retrieves the second member from the provided pair.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param pr The pair instance.
	@param out The pointer in which to store the result.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_pairGetSecond( lasso_request_t token, lasso_type_t pr, lasso_type_t * out );
/**
	Sets the first member of the provided pair.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param pr The pair instance.
	@param frst The instance to use as the first member. The instance will be referenced.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_pairSetFirst( lasso_request_t token, lasso_type_t pr, lasso_type_t frst );
/**
	Sets the second member of the provided pair.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param pr The pair instance.
	@param scnd The instance to use as the second member. The instance will be referenced.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_pairSetSecond( lasso_request_t token, lasso_type_t pr, lasso_type_t scnd );
/** @} */
/**
	@brief Adds a member to a type instance.
	
	If the new member is a tag, it will be added to the tag members for the type. Otherwise, the new member will be added as a data member.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param to The type instance to which the new member will be added.
	@param named The name for the new member.
	@param member The new member to add.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAddMember( lasso_request_t token, lasso_type_t to, const char * named, lasso_type_t member );
/**
	@brief Adds a member to a type instance.
	
	If the new member is a tag, it will be added to the tag members for the type. Otherwise, the new member will be added as a data member.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param to The type instance to which the new member will be added.
	@param named The name for the new member.
	@param member The new member to add.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAddMemberW( lasso_request_t token, lasso_type_t to, const UChar * named, lasso_type_t member );
/**
	@brief Adds a tag member to a type instance.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param to The type instance to which the new tag member will be added.
	@param named The name for the new member.
	@param member The new tag member to add.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAddTagMember( lasso_request_t token, lasso_type_t to, const char * named, lasso_type_t member );
/**
	 @brief Adds a tag member to a type instance.
	 
	 @param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	 @param to The type instance to which the new tag member will be added.
	 @param named The name for the new member.
	 @param nativeTagFunction The function add.
	 @return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
 */
LCAPICALL osError lasso_typeAddTagMember2( lasso_request_t token, lasso_type_t to, const char * named, lasso_tag_func nativeTagFunction );
/**
	@brief Adds a data member to a type instance.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param to The type instance to which the new data member will be added.
	@param named The name for the new member.
	@param member The new data member to add.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAddDataMember( lasso_request_t token, lasso_type_t to, const char * named, lasso_type_t member );
/**
	@brief Adds a tag member to a type instance.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param to The type instance to which the new tag member will be added.
	@param named The name for the new member.
	@param member The new tag member to add.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAddTagMemberW( lasso_request_t token, lasso_type_t to, const UChar * named, lasso_type_t member );
/**
	 @brief Adds a tag member to a type instance.
	 
	 @param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	 @param to The type instance to which the new tag member will be added.
	 @param named The name for the new member.
	 @param nativeTagFunction The function to add.
	 @return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
 */
LCAPICALL osError lasso_typeAddTagMember2W( lasso_request_t token, lasso_type_t to, const UChar * named, lasso_tag_func nativeTagFunction );
/**
	@brief Adds a data member to a type instance.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param to The type instance to which the new data member will be added.
	@param named The name for the new member.
	@param member The new data member to add.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeAddDataMemberW( lasso_request_t token, lasso_type_t to, const UChar * named, lasso_type_t member );
/**
	@brief Retrieves a data member from a type instance.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param from The type instance in which to search.
	@param named The name of the member to look for.
	@param out A pointer to a type instance in which to store the found member.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeGetDataMember( lasso_request_t token, lasso_type_t from, const char * named, lasso_type_t * out );
/**
	@brief Retrieves a data member from a type instance.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param from The type instance in which to search.
	@param named The name of the member to look for.
	@param out A pointer to a type instance in which to store the found member.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeGetDataMemberW( lasso_request_t token, lasso_type_t from, const UChar * named, lasso_type_t * out );

/**
 @brief Sets a data member of a type instance.
 
 @param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
 @param from The type instance in which to search.
 @param named The name of the member to look for.
 @param to A pointer to a type instance. The new value for the data member.
 @return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
 */
LCAPICALL osError lasso_typeSetDataMember( lasso_request_t token, lasso_type_t from, const char * named, lasso_type_t to );
/**
 @brief Sets a data member of a type instance.
 
 @param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
 @param from The type instance in which to search.
 @param named The name of the member to look for.
 @param to A pointer to a type instance. The new value for the data member.
 @return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
 */
LCAPICALL osError lasso_typeSetDataMemberW( lasso_request_t token, lasso_type_t from, const UChar * named, lasso_type_t to );


/**
	@brief Allows storage of an opaque pointer value.
	
	The pointer member is given and name and is stored as an integer in the type instance.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param self The type instance in which to add the pointer member.
	@param name The name for the new member.
	@param data The pointer value which will be added.
	@param dtor A pointer to a function which will be called when the member is destroyed.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_setPtrMember(lasso_request_t token, lasso_type_t self, const char * name, void * data, void (*dtor)(void*));
/**
	 @brief Allows storage of an opaque pointer value.
	 
	 The pointer member is given and name and is stored as an integer in the type instance.
	 
	 @param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	 @param from The type instance in which to add the pointer member.
	 @param name The name for the new member.
	 @param data The pointer value which will be added.
	 @param dtor A pointer to a function which will be called when the member is destroyed.
	 @param copyFunc A pointer to a function which will be called when the member is copied.
	 @return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_setPtrMember2(lasso_request_t token, lasso_type_t from, const char * name, void * data, void (*dtor)(void * obj), void * (*copyFunc)(void * obj));
/**
	@brief Allows storage of an opaque pointer value.
	
	The pointer member is given and name and is stored as an integer in the type instance.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param self The type instance in which to add the pointer member.
	@param name The name for the new member.
	@param data The pointer value which will be added.
	@param dtor A pointer to a function which will be called when the member is destroyed.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_setPtrMemberW(lasso_request_t token, lasso_type_t self, const UChar * name, void * data, void(*dtor)(void*));
/**
	 @brief Allows storage of an opaque pointer value.
	 
	 The pointer member is given and name and is stored as an integer in the type instance.
	 
	 @param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	 @param from The type instance in which to add the pointer member.
	 @param named The name for the new member.
	 @param data The pointer value which will be added.
	 @param dtor A pointer to a function which will be called when the member is destroyed.
	 @param copyFunc A pointer to a function which will be called when the member is copied.
	 @return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_setPtrMember2W(lasso_request_t token, lasso_type_t from, const UChar * named, void * data, void (*dtor)(void * obj), void * (*copyFunc)(void * obj));
/**
	@brief Retrieves a previously added pointer value.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param self The type instance from which to retrieve.
	@param name The name of the previously added pointer value.
	@param data A pointer to a void * in which to store the found value.s
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getPtrMember(lasso_request_t token, lasso_type_t self, const char * name, void ** data);
/**
	@brief Retrieves a previously added pointer value.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param self The type instance from which to retrieve.
	@param name The name of the previously added pointer value.
	@param data A pointer to a void * in which to store the found value.s
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getPtrMemberW(lasso_request_t token, lasso_type_t self, const UChar * name, void ** data);
/**
	@brief Returns the type instance that the current tag call was a member of.
	
	This is used in member tags of custom types to return the target of the current call.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param self A pointer in which to store the resulting type instance.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getTagSelf ( lasso_request_t token, lasso_type_t * self );
/**
	@brief Returns the name of the target type.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param target The type instance to get the name of.
	@param outName The resulting name which will be stored in the \a name portion of the struct.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/	
LCAPICALL osError lasso_typeGetName( lasso_request_t token, lasso_type_t target, auto_lasso_value_t * outName );
/**
	@brief Returns the name of the target type.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param target The type instance to get the name of.
	@param outName The resulting name which will be stored in the \a name portion of the struct.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/	
LCAPICALL osError lasso_typeGetNameW( lasso_request_t token, lasso_type_t target, auto_lasso_value_w_t * outName );
/**
	@brief Tests to see if a type is an instance of another type.
	
	Test by type code.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param target The target instance to test.
	@param type The type code to test for.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeIsA( lasso_request_t token, lasso_type_t target, LP_TypeDesc type );
/**
	@brief Tests to see if a type is an instance of another type.
	
	Test by type name.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param target The target instance to test.
	@param typeName The type name to test for.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeIsA2( lasso_request_t token, lasso_type_t target, const char * typeName );
/**
	@brief Tests to see if a type is an instance of another type.
	
	Test by type name.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param target The target instance to test.
	@param typeName The type name to test for.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeIsA2W( lasso_request_t token, lasso_type_t target, const UChar * typeName );
/**
	@brief Tests to see if a type is an instance of another type.
	
	Test based on another type instance.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param target The target instance to test.
	@param type The type name to test for.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_typeIsA3( lasso_request_t token, lasso_type_t target, lasso_type_t type );

/**
	@brief Specifies the return value of the current LCAPI tag call.

	Any type can be returned. A tag can only have one return value. Setting another return value will overwrite the previous.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param value The value to return.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_returnTagValue( lasso_request_t token, lasso_type_t value );
/**
	@brief Return a boolean value from the current LCAPI tag call.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param b The boolean value to return.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_returnTagValueBoolean( lasso_request_t token, bool b );
/**
	@brief Return an integer value from the current LCAPI tag call.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param i The integer value to return.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_returnTagValueInteger( lasso_request_t token, int64_t i );
/**
	@brief Return a string value from the current LCAPI tag call.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param p The character data to be returned. The data should be in UTF-8 encoding.
	@param l The length of the character data to return.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_returnTagValueString( lasso_request_t token, const char * p, int l );
/**
	@brief Return a string value from the current LCAPI tag call.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param p The character data to be returned.
	@param l The length of the character data to return.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_returnTagValueStringW( lasso_request_t token, const UChar * p, int l );
/**
	@brief Return a decimal value from the current LCAPI tag call.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param d The double value to return.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_returnTagValueDecimal( lasso_request_t token, double d );
/**
	@brief Return binary data from the current LCAPI tag call.
	
	When using this, the result of the LCAPI call will be a bytes type.
	This can be called as many times as needed and new data will be appended to any previous data.

	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param data The binary data to return.
	@param length The number of bytes to return.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_returnTagValueBytes( lasso_request_t token, const char * data, int length );
/**
	@brief Retrieves a parameter that was passed to the LCAPI tag call.

	The parameter index is zero based.

	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param paramIndex The zero based index of the desired parameter.
	@param result The resulting parameter value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_getTagParam2( lasso_request_t token, int paramIndex, lasso_type_t * result );

/**
	@brief Retrieves a parameter that was passed to the LCAPI tag call.

	The parameter must have been specified by keyword.

	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param paramName The keyword of the desired parameter.
	@param result The resulting parameter value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_findTagParam2( lasso_request_t token, const char * paramName, lasso_type_t * result );
/**
	@brief Retrieves a parameter that was passed to the LCAPI tag call.

	The parameter must have been specified by keyword.

	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param paramName The keyword of the desired parameter.
	@param result The resulting parameter value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_findTagParam2W( lasso_request_t token, const UChar * paramName, lasso_type_t * result );
/**
	@brief Register a constant value.
	
	Constants can be called just like tags, but the resulting value will only have a single instance and will have both its type and value frozen.
	This is usually called at the same time that tags are registered.
	
	@param namespaceName The namespace for the constant.
	@param name The name for the constant.
	@param val The value for the constant.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_registerConstant2( const char * namespaceName, const char * name, lasso_type_t val );
/**
	@brief Register a constant value.
	
	Constants can be called just like tags, but the resulting value will only have a single instance and will have both its type and value frozen.
	This is usually called at the same time that tags are registered.
	
	@param namespaceName The namespace for the constant.
	@param name The name for the constant.
	@param val The value for the constant.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_registerConstant2W( const UChar * namespaceName, const UChar * name, lasso_type_t val );
/**
	@brief Register a constant value.
	
	Constants can be called just like tags, but the resulting value will only have a single instance and will have both its type and value frozen.
	This is usually called at the same time that tags are registered.
	
	@param name The name for the constant.
	@param val The value for the constant.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_registerConstant( const char * name, lasso_type_t val );
/**
	@brief Register a constant value.
	
	Constants can be called just like tags, but the resulting value will only have a single instance and will have both its type and value frozen.
	This is usually called at the same time that tags are registered.
	
	@param name The name for the constant.
	@param val The value for the constant.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_registerConstantW( const UChar * name, lasso_type_t val );
/**
	@brief Fetches the name of the tag that triggered this call.
	
	For example, in the case of: [my_tag: ...] the resulting value would be "my_tag".
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param result The resulting tag name which will be placed in the \a name portion of the struct.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getTagName( lasso_request_t token, auto_lasso_value_t * result);
/**
	@brief Fetches the name of the tag that triggered this call.
	
	For example, in the case of: [my_tag: ...] the resulting value would be "tag".
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param result The resulting tag name which will be placed in the \a name portion of the struct.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getTagNameW( lasso_request_t token, auto_lasso_value_w_t * result);
/**
	@brief Fetches the number of parameters that were passed to the tag.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param result A pointer in which to store the result.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getTagParamCount( lasso_request_t token, int * result );
/**
	@brief Fetches the indicated parameter that was sent to the tag.
	
	Parameter indexes start at zero. If the parameter was specified as a keyword/value pair, the keyword will be placed in the \a name portion of the struct and the value in the \a data portion.
	If the parameter was provided without a keyword, the \a name portion of the struct will be NULL.
	
	All parameters values will be converted to string, regardless of the original type.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param paramIndex The zero based index of the desired parameter.
	@param result The resulting parameter data.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getTagParam( lasso_request_t token, int paramIndex, auto_lasso_value_t * result );
/**
	@brief Fetches the indicated parameter that was sent to the tag.
	
	Parameter indexes start at zero. If the parameter was specified as a keyword/value pair, the keyword will be placed in the \a name portion of the struct and the value in the \a data portion.
	If the parameter was provided without a keyword, the \a name portion of the struct will be NULL.
	
	All parameters values will be converted to string, regardless of the original type.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param paramIndex The zero based index of the desired parameter.
	@param result The resulting parameter data.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getTagParamW( lasso_request_t token, int paramIndex, auto_lasso_value_w_t * result );
/**
	@brief Returns osErrNoErr if the param WAS defined.
	
	Any other result means it wasn't defined. Only parameters specified with a keyword should be searched for.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param paramName The name of the keyworded parameter to search for.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_tagParamIsDefined( lasso_request_t token, const char * paramName );
/**
	@brief Returns osErrNoErr if the param WAS defined.
	
	Any other result means it wasn't defined. Only parameters specified with a keyword should be searched for.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param paramName The name of the keyworded parameter to search for.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_tagParamIsDefinedW( lasso_request_t token, const UChar * paramName );
/**
	@brief Retrieves a parameter that was passed to the LCAPI tag call.

	The parameter must have been specified by keyword. 
	The keyword will be placed in the \a name portion of the struct and the value in the \a data portion. 
	If only a keyword was provided, the \a data portion will be NULL.

	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param paramName The keyword of the desired parameter.
	@param result The resulting parameter value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_findTagParam( lasso_request_t token, const char * paramName, auto_lasso_value_t * result );
/**
	@brief Retrieves a parameter that was passed to the LCAPI tag call.

	The parameter must have been specified by keyword. 
	The keyword will be placed in the \a name portion of the struct and the value in the \a data portion. 
	If only a keyword was provided, the \a data portion will be NULL.

	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param paramName The keyword of the desired parameter.
	@param result The resulting parameter value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_findTagParamW( lasso_request_t token, const UChar * paramName, auto_lasso_value_w_t * result );

LCAPICALL osError lasso_setVariable( lasso_request_t token, const char * named, const char * value, int index );
LCAPICALL osError lasso_setVariableW( lasso_request_t token, const UChar * named, const UChar * value, int index );

/**
	@name Datasource Module Functions
*/
/** @{ */
/**
	@brief Adds a datasource result value.
	
	Datasource actions which require returning multiple values can use this to add those values.
	For example, this call can be used to add the name of a datasource that the module services or the names of the tables in a particular datasource.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param data The data to add. Data must be in ISO8859-1 encoding.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_addDataSourceResult( lasso_request_t token, const char * data );
/**
	@brief Adds a datasource result value.
	
	Datasource actions which require returning multiple values can use this to add those values.
	For example, this call can be used to add the name of a datasource that the module services or the names of the tables in a particular datasource.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param data The data to add. Data must be in UTF-8 encoding.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_addDataSourceResultUTF8( lasso_request_t token, const char * data );
/**
	@brief Gets the currently specified database name and associated data.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param t The resulting database name.
	@param useHostDefault The setting which specifies whether the hosts username/password combo should be used in the absence of a database specific combo.
	@param usernamepassword The username/password combo for the database.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getDataSourceName( lasso_request_t token, auto_lasso_value_t * t, bool * useHostDefault, auto_lasso_value_t * usernamepassword );
/**
	@brief Gets the currently specified table name.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param t The resulting table name which will be placed in the \a name portion of the struct.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getTableName( lasso_request_t token, auto_lasso_value_t * t );
/**
	@brief Gets the currently specified table encoding. This is the encoding as set in SiteAdmin for the current table.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param t The resulting table encoding which will be placed in the \a name portion of the struct.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getTableEncoding( lasso_request_t token, auto_lasso_value_t * t );
/**
	@brief Gets the currently specified schema name.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param schema The resulting schema name which will be placed in the \a name portion of the struct.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getSchemaName( lasso_request_t token, auto_lasso_value_t * schema);
/**
	@brief Returns the host that maintains the current database.
	
	The host name will be placed in the \a name portion of the struct while the port will be placed in the \a data portion.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param host The resulting host data.
	@param usernamepassword The resulting username/password combo.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getDataHost( lasso_request_t token, auto_lasso_value_t * host, auto_lasso_value_t * usernamepassword );
/**
	 @brief Returns the id of the host that maintains the current database.
	 
	 @param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	 @param outId The resulting host id.
	 @return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getDataHostID( lasso_request_t token, int * outId );
/**
	@brief Returns the host that maintains the current database.
	
	The host name will be placed in the \a name portion of the struct while the port will be placed in the \a data portion.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param host The resulting host data.
	@param defaultSchema The resulting default schema name which will be placed in the \a name portion of the struct.
	@param usernamepassword The resulting username/password combo.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getDataHost2( lasso_request_t token, auto_lasso_value_t * host, auto_lasso_value_t * defaultSchema, auto_lasso_value_t * usernamepassword );
/**
	@brief Returns the "extra" information associated with the current host.
	
	The data, if any, will be placed in the \a name portion of the struct.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param data The resulting extra  host data.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getDataHostExtra( lasso_request_t token, auto_lasso_value_t * data );
/**
	@brief Indicates if the host is dynamic.
		
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param wasDyn Will be set to true if the host was dynamic and false if it was not.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getDataHostIsDynamic( lasso_request_t token, bool * wasDyn );

/**
	@brief The number of rows that should be skipped in the found set.
	
	-skiprecords
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param recs The resulting -skiprecords value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_getSkipRows( lasso_request_t token, int * recs );
/**
	@brief The maximum number of rows in the found set to return.
	
	-maxrecords
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param recs The resulting -maxrecords value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_getMaxRows( lasso_request_t token, int * recs );
/**
	@brief Sets the currently specified record id (FileMaker specific).
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param id The numeric row id.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_setRowID( lasso_request_t token, int id );
	
LCAPICALL osError lasso_setRowID2( lasso_request_t token, unsigned long long id );
LCAPICALL osError lasso_getRowID2( lasso_request_t token, unsigned long long * id );
/**
	 @brief Gets the currently specified record id (FileMaker specific).
	 
	 @param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	 @param id The numeric row id.
	 @return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getRowID( lasso_request_t token, int * id );
/**
	@brief Gets the name and the value of the currently specified primary key column.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param v The resulting key name and value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_getPrimaryKeyColumn( lasso_request_t token, auto_lasso_value_t * v );
/**
	@brief Gets the name and the value of the primary key column specified by index.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param index The zero based index.
	@param v The resulting key name and value.
	@param desc A pointer to a LP_TypeDesc in which to place the original type of the input data.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_getPrimaryKeyColumn2( lasso_request_t token, int index, auto_lasso_value_t * v, LP_TypeDesc * desc );
/**
	@brief Gets the name and the value of the primary key column specified by index, preserving the original type.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param index The zero based index.
	@param keyName The name of the key field will be stored in the \a name member.
	@param type The value portion of the key field in its original type.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_getPrimaryKeyColumn3( lasso_request_t token, int index, auto_lasso_value_t * keyName, lasso_type_t * type );
/**
	@brief Gets the number of key columns.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param count A pointer to an int in which the number of key columns will be placed.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_getPrimaryKeyColumnCount( lasso_request_t token, int * count );
/**
	@brief Gets the number of input columns for this database action.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param count The resulting number of input columns.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_getInputColumnCount( lasso_request_t token, int * count );
/**
	@brief Gets the number of sort columns for this database action.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param count The resulting number of sort columns.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_getSortColumnCount( lasso_request_t token, int * count );
/**
	@brief Gets an individual input column by index.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param num the zero based index of the column to retrieve.
	@param v The resulting value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_getInputColumn( lasso_request_t token, int num, auto_lasso_value_t * v );
/**
	@brief Gets an individual input column by index.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param num the zero based index of the column to retrieve.
	@param v The resulting value.
	@param desc A pointer to a LP_TypeDesc in which to place the original type of the input data.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_getInputColumn2( lasso_request_t token, int num, auto_lasso_value_t * v, LP_TypeDesc * desc );
/**
	@brief Gets an individual input column by index, preserving the original type.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param num the zero based index of the column to retrieve.
	@param colName The name of the column will be stored in the \a name member.
	@param type The value portion of the input column in its original type.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_getInputColumn3( lasso_request_t token, int num, auto_lasso_value_t * colName, lasso_type_t * type );
/**
	@brief Gets an individual sort column by index.

	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param num the zero based index of the sort column to retrieve.
	@param v The resulting value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_getSortColumn( lasso_request_t token, int num, auto_lasso_value_t * v );
/**
	@brief Gets an individual input column by name.

	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param name the name of the input column to retrieve.
	@param value The resulting value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_findInputColumn( lasso_request_t token, const char * name, auto_lasso_value_t * value );
/**
	@brief Gets an individual input column by name.

	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param name the name of the input column to retrieve.
	@param value The resulting value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_findInputColumnW( lasso_request_t token, const UChar * name, auto_lasso_value_t * value );
/**
	@brief Gets the logical operator for this database action.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param op The resulting operator code. Operator codes are declared at the top of this file.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_getLogicalOp( lasso_request_t token, LP_TypeDesc * op );
/**
	@brief Gets the number of return columns for this action.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param count The resulting number of return columns.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_getReturnColumnCount( lasso_request_t token, int * count );
/**
	@brief Gets an individual return column by index.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param num The zero based index of the return column to retrieve.
	@param v The resulting value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_getReturnColumn( lasso_request_t token, int num, auto_lasso_value_t * v );
/**
	@brief Adds information about a particular column. 
	
	Column information should be added in the order in which the columns occur inthe database.
	Column information should be added no matter what the action is. For the show action, only column information is added.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param name The column name.
	@param nullOK IS a null value for this column ok?
	@param type The data type for the column as represented by a type code. osConfig.h.
	@param protection The code for the column's protection. osConfig.h.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_addColumnInfo( lasso_request_t token, const char * name, int nullOK, LP_TypeDesc type, LP_TypeDesc protection );
/**
	@brief Adds information, including valuelists, about a particular column.
	
	Column information should be added in the order in which the columns occur inthe database.
	Column information should be added no matter what the action is. For the show action, only column information is added.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param name The column name.
	@param nullOK IS a null value for this column ok?
	@param type The data type for the column as represented by a type code. osConfig.h.
	@param protection The code for the column's protection. osConfig.h.
	@param valueList An array of strings for the column's valuelist.
	@param countValueList The number of values in the value list.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_addColumnInfo2( lasso_request_t token, const char * name, int nullOK, LP_TypeDesc type, LP_TypeDesc protection, const char ** valueList, int countValueList );
/**
	@brief Add the column data for the next result row.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param columns An array of column values.
	@param dataSizes An array of column value lengths. Every column in \a columns should have an associated length.
	@param numColumns The number of items in the \a columns and \a dataSizes arrays.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_addResultRow( lasso_request_t token, const char ** columns, unsigned long * dataSizes, int numColumns );
/**
	@brief Add the column data for the next result row. Column data is represented as Lasso types.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param cols An array of column values.
	@param num The number of items in the \a cols array.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_addResultRow2( lasso_request_t token, lasso_type_t* cols, int num );

/**
	@brief Adds a new result set to the datasource results.
	
	At the onset of each LCAPI datasource call, there is an initial blank result set created. This is the first result set. 
	After a datasource has added all the data for the first result set, if there is a second result set, the datasource should call
	lasso_addResultSet to start a new set and then proceed to populate the column information and data for that set as normal. 
	This should be repeated for each result set. lasso_addResultSet should be called for each result set after the first. 
	Calling lasso_addResultSet before populating the first result set will result in the first set being empty.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_addResultSet( lasso_request_t token );
/**
	@brief Sets the number of rows found in the query. 
	
	This will not always be the same value as the number of rows added with the lasso_addResultRow call as the skip recs and max recs parameters effect that number.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param num the number of found rows.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_setNumRowsFound( lasso_request_t token, int num );
/**
	@brief Returns the name the current datasource module was registered with.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param val The resulting datasource module name which will be palced in the \a name portion of the struct.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_getDataSourceModuleName( lasso_request_t token, auto_lasso_value_t * val );
/**
	@brief Allows a datasource to set a prepared statement pointer.

	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param ptr The pointer value to set for later retrieval.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_setDSPreparedPtr( lasso_request_t token, void * ptr );
/**
	@brief Allows a datasource to get a previously set prepared statement pointer.

	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param ptr The pointer, if it has been presiously set, will be placed in this parameter.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getDSPreparedPtr( lasso_request_t token, void ** ptr );

/**
	@brief Called to access the current datasource connection.
	
	Datasource connections are set using the lasso_setDSConnection function.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param conn A pointer in which to store the result.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL osError lasso_getDSConnection(lasso_request_t token, lasso_dsconnection_t * conn);
/**
	@brief Called to set the current connection for the datasource.
	
	May recurse to deliver the datasourceCloseConnection message if there is already a valid lasso_dbconnection_t.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param conn The connection value to set.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_setDSConnection(lasso_request_t token, lasso_dsconnection_t conn);
/**
	@brief Provides access to the "user data" that was set when the datasorce was registered.
 
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param outPtr The pointer in which the user data is returned.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.
*/
LCAPICALL osError lasso_getDSUserData(lasso_request_t token, void ** outPtr);

/**
	@brief Called to set the statement for the current action.
	
	Datasources must call this to support the action_statement tag.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param stat The statement value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_setActionStatement(lasso_request_t token, const char * stat);
/**
	@brief Called to set the statement for the current action.
	
	Datasources must call this to support the action_statement tag.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param stat The statement value.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_setActionStatementW(lasso_request_t token, const UChar * stat);
/**
	@brief Used to check for the -statementonly inline param.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param out A pointer in which to store the result.	
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL osError lasso_getIsStatementOnly(lasso_request_t token, bool * out);
/** @} */

const int kMaxPathLen = 1024;
typedef char osPathname[1024];

/**
	@brief This function is a no-op in Lasso 9.

	@param inInternalPath The internal pathname.
	@param outPlatformPath The resulting platform specific pathname.
*/
LCAPICALL void lasso_getPlatformSpecificPath( const char* inInternalPath, osPathname outPlatformPath );

/**
	@brief This function is a no-op in Lasso 9.
	
	@param inPlatformPath the platform specific path name.
	@param outInternalPath The resulting internal pathname.
*/
LCAPICALL void lasso_getInternalPath( const char* inPlatformPath, osPathname outInternalPath );

/**
	@brief This function is a no-op in Lasso 9.
	
	If the path is already from the root it won't be changed.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param inRelativePath The relative path.
	@param outFullyQualified the resulting fully qualified path.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.		
*/
LCAPICALL void lasso_fullyQualifyPath( lasso_request_t token, const char * inRelativePath, osPathname outFullyQualified );

/**
	@brief This function is a no-op in Lasso 9.

	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param inPath The path to resolve.
	@param outFullPath The resulting full path.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.			
*/
LCAPICALL void lasso_resolvePath( lasso_request_t token, const char * inPath, osPathname outFullPath );

/**
	@brief This function is a no-op in Lasso 9.

	Resulting path must name an item within the web-root or a blank string is returned.
	
	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param inPath The path to resolve.
	@param outFullPath The resulting full path.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.			
*/
LCAPICALL void lasso_resolveIncludePath( lasso_request_t token, const char * inPath, osPathname outFullPath );

/**
	@brief This function is a no-op in Lasso 9.
	
	Takes a path, such as one which may have been passed to any Lasso tag, and converts it into a full platform specific path name.

	@param token The request token which should be passed to subsequent LCAPI functions for the duration of the current service function call.
	@param relativeOrFullInternalPath The input path to convert.
	@param fullPlatformPath The resulting converted path.
	@return If the function succeeds, osErrNoErr is returned, otherwise, an error code indicating the problem is returned.	
*/
LCAPICALL void lasso_internalToFullPlatformPath( lasso_request_t token, const char * relativeOrFullInternalPath, osPathname fullPlatformPath );

/**
	@brief Returns true if the path begins with a forward slash.
	
	@param path The path to test.
	@return True if the given internal path is a full path from the filesystem root.
*/
LCAPICALL bool lasso_isFullInternalPath(const char * path);
/**
	@name Logging Functions
*/
/** @{ */
/**
	@brief The various log levels.
	
	Each log level can be configured to go to zero or more destinations.
*/
typedef enum log_level_t
{
	LOG_LEVEL_ALWAYS, /**< Always printed to window (only). Used internally in a few places. */
    
} log_level_t;

/**
	@brief Log a message to stdout.
*/
LCAPICALL osError lasso_log( log_level_t msgLevel, const char * fmt, ... );
/** @} */

}

#endif
